/*
 * Copyright 2013 Magnus Hjelmblom, University of Gvle, Sweden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package se.hig.estates;

import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import se.hig.nrts.Act;
import se.hig.nrts.Agent;
import se.hig.nrts.GenericComponent;
import se.hig.nrts.State;
import se.hig.nrts.client.NRTSClientPanel;
import se.hig.nrts.norms.Consequence;
import se.hig.nrts.norms.Ground;
import se.hig.nrts.norms.MoveOperator;
import se.hig.nrts.norms.SitCondition;
import se.hig.nrts.norms.StateCondition;
import se.hig.nrts.norms.TypeOperator;
import se.sics.prologbeans.PBTerm;

public class EstatesFrame extends JFrame implements Observer {
	/**
	 * 
	 */
	private static final long serialVersionUID = 8304003468713100963L;
	protected Box centerBox = Box.createVerticalBox();
	protected Box southBox = Box.createHorizontalBox();
	private NRTSClientPanel controlPanel = null;
	private JButton showPermissibleActsBtn = new JButton("Permissible acts");
	private JButton showResultingStateBtn = new JButton("Resulting state");
	private List<Act> permissibleActs = null;
	private int inconsistencyCounter = 0;

	private final String [] WHITEACRE = {"whiteacre"};
	private final String [] BLACKACRE = {"blackacre"};

	public EstatesFrame(String host, int port, int rooms)
			throws HeadlessException {
		super("Estates System @ " + host + ":" + port);
		/*
		 * --- CONTROL PANEL ---
		 */
		controlPanel = new NRTSClientPanel(host, port);
		String agentID0 = "alice";
		Agent alice = controlPanel.addAgent(agentID0);
		String agentID1 = "bob";
		Agent bob = controlPanel.addAgent(agentID1);
		String agentID2 = "charlie";
		Agent charlie = controlPanel.addAgent(agentID2);
		addEstate(WHITEACRE[0],agentID0,"main_building_painted_white","no_cows","not_fenced");
		addEstate(BLACKACRE[0],agentID1,"main_building_painted_black","no_cows","not_fenced");
		
		// --- Create ground-set ---
		Map<String,StateCondition> groundSet = controlPanel.getGroundSet();
		final String OWNER = "owner";
		StateCondition owner = StateCondition.createStateCondition(OWNER,1,1);
		controlPanel.addConditionToGroundSet(OWNER,owner);
		SitCondition m1OwnerToWhiteacre = 
				SitCondition.createSitCond(MoveOperator.M1,OWNER,WHITEACRE);
		Ground M1_owner_to_whiteacre = Ground.createElementaryGround(m1OwnerToWhiteacre,groundSet);

		SitCondition m1OwnerToBlackacre = 
				SitCondition.createSitCond(MoveOperator.M1,OWNER,BLACKACRE);
		Ground M1_owner_to_blackacre = Ground.createElementaryGround(m1OwnerToBlackacre,groundSet);

		Ground not_M1_owner_to_whiteacre = Ground.createNegatedGround(M1_owner_to_whiteacre);
		Ground not_M1_owner_to_blackacre = Ground.createNegatedGround(M1_owner_to_blackacre);

		// --- Create consequence-set ---
		final String MAIN_BUILDING_PAINTED_WHITE = "main_building_painted_white";
		StateCondition main_building_painted_white = 
				StateCondition.createStateCondition(MAIN_BUILDING_PAINTED_WHITE,1,0);
		controlPanel.addConditionToConsequenceSet(MAIN_BUILDING_PAINTED_WHITE,main_building_painted_white);
		Map<String,StateCondition> consequenceSet = controlPanel.getConsequenceSet();
		SitCondition p1MainBuildingOnWhiteacrePaintedWhite = 
				SitCondition.createSitCond(TypeOperator.P1,MAIN_BUILDING_PAINTED_WHITE,WHITEACRE);
		Consequence P1_main_building_on_whiteacre_painted_white = 
				Consequence.createElementaryConsequence(p1MainBuildingOnWhiteacrePaintedWhite,consequenceSet);

		SitCondition p1MainBuildingOnBlackacrePaintedWhite = 
				SitCondition.createSitCond(TypeOperator.P1,MAIN_BUILDING_PAINTED_WHITE,BLACKACRE);
		Consequence P1_main_building_on_blackacre_painted_white = 
				Consequence.createElementaryConsequence(p1MainBuildingOnBlackacrePaintedWhite,consequenceSet);

		SitCondition p61MainBuildingOnWhiteacrePaintedWhite = 
				SitCondition.createSitCond(TypeOperator.P61,MAIN_BUILDING_PAINTED_WHITE,WHITEACRE);
		Consequence P61_main_building_on_whiteacre_painted_white = 
				Consequence.createElementaryConsequence(p61MainBuildingOnWhiteacrePaintedWhite,consequenceSet);
		
		SitCondition p61MainBuildingOnBlackacrePaintedWhite = 
				SitCondition.createSitCond(TypeOperator.P61,MAIN_BUILDING_PAINTED_WHITE,BLACKACRE);
		Consequence P61_main_building_on_blackacre_painted_white = 
				Consequence.createElementaryConsequence(p61MainBuildingOnBlackacrePaintedWhite,consequenceSet);

//		final String COWS_ENTERED = "cows_entered";
//		StateCondition cows_entered = 
//				StateCondition.createStateCondition(COWS_ENTERED,2,0);
//		controlPanel.addConditionToConsequenceSet(COWS_ENTERED,cows_entered);
//		SitCondition p1CowsEnteredFromWhiteacreToBlackacre = 
//				SitCondition.createSitCond(TypeOperator.P1,COWS_ENTERED,
//						controlPanel.splitParamsString("whiteacre,blackacre"));
//		Consequence P1_cows_entered_from_whiteacre_to_blackacre = 
//				Consequence.createElementaryConsequence(p1CowsEnteredFromWhiteacreToBlackacre,consequenceSet);
//		
//		SitCondition p41CowsEnteredFromWhiteacreToBlackacre = 
//				SitCondition.createSitCond(TypeOperator.P41,COWS_ENTERED,
//						controlPanel.splitParamsString("whiteacre,blackacre"));
//		Consequence P41_cows_entered_from_whiteacre_to_blackacre = 
//				Consequence.createElementaryConsequence(p41CowsEnteredFromWhiteacreToBlackacre,consequenceSet);
//
//		SitCondition p61CowsEnteredFromWhiteacreToBlackacre = 
//				SitCondition.createSitCond(TypeOperator.P61,COWS_ENTERED,
//						controlPanel.splitParamsString("whiteacre,blackacre"));
//		Consequence P61_cows_entered_from_whiteacre_to_blackacre = 
//				Consequence.createElementaryConsequence(p61CowsEnteredFromWhiteacreToBlackacre,consequenceSet);
//
//		SitCondition p7CowsEnteredFromWhiteacreToBlackacre = 
//				SitCondition.createSitCond(TypeOperator.P7,COWS_ENTERED,
//						controlPanel.splitParamsString("whiteacre,blackacre"));
//		Consequence P7_cows_entered_from_whiteacre_to_blackacre = 
//				Consequence.createElementaryConsequence(p7CowsEnteredFromWhiteacreToBlackacre,consequenceSet);
//
//		SitCondition p1CowsEnteredFromBlackacreToWhiteacre = 
//				SitCondition.createSitCond(TypeOperator.P1,COWS_ENTERED,
//						controlPanel.splitParamsString("blackacre,whiteacre"));
//		Consequence P1_cows_entered_from_blackacre_to_whiteacre = 
//				Consequence.createElementaryConsequence(p1CowsEnteredFromBlackacreToWhiteacre,consequenceSet);
//		
//		SitCondition p41CowsEnteredFromBlackacreToWhiteacre = 
//				SitCondition.createSitCond(TypeOperator.P41,COWS_ENTERED,
//						controlPanel.splitParamsString("blackacre,whiteacre"));
//		Consequence P41_cows_entered_from_blackacre_to_whiteacre = 
//				Consequence.createElementaryConsequence(p41CowsEnteredFromBlackacreToWhiteacre,consequenceSet);
//
//		SitCondition p61CowsEnteredFromBlackacreToWhiteacre = 
//				SitCondition.createSitCond(TypeOperator.P61,COWS_ENTERED,
//						controlPanel.splitParamsString("blackacre,whiteacre"));
//		Consequence P61_cows_entered_from_blackacre_to_whiteacre = 
//				Consequence.createElementaryConsequence(p61CowsEnteredFromBlackacreToWhiteacre,consequenceSet);
//
//		SitCondition p7CowsEnteredFromBlackacreToWhiteacre = 
//				SitCondition.createSitCond(TypeOperator.P7,COWS_ENTERED,
//						controlPanel.splitParamsString("blackacre,whiteacre"));
//		Consequence P7_cows_entered_from_blackacre_to_whiteacre = 
//				Consequence.createElementaryConsequence(p7CowsEnteredFromBlackacreToWhiteacre,consequenceSet);

		final String ERECTED_FENCE_AROUND = "erected_fence_around";
		StateCondition erected_fence_around = 
				StateCondition.createStateCondition(ERECTED_FENCE_AROUND,2,0);
		controlPanel.addConditionToConsequenceSet(ERECTED_FENCE_AROUND,erected_fence_around);
		SitCondition p1ErectedFenceAround = 
				SitCondition.createSitCond(TypeOperator.P1,ERECTED_FENCE_AROUND,
						controlPanel.splitParamsString("whiteacre,blackacre"));
		Consequence P1_erected_fence_around = 
				Consequence.createElementaryConsequence(p1ErectedFenceAround,consequenceSet);
		
		SitCondition p41ErectedFenceAround = 
				SitCondition.createSitCond(TypeOperator.P41,ERECTED_FENCE_AROUND,
						controlPanel.splitParamsString("whiteacre,blackacre"));
		Consequence P41_erected_fence_around = 
				Consequence.createElementaryConsequence(p41ErectedFenceAround,consequenceSet);

		SitCondition p61ErectedFenceAround = 
				SitCondition.createSitCond(TypeOperator.P61,ERECTED_FENCE_AROUND,
						controlPanel.splitParamsString("whiteacre,blackacre"));
		Consequence P61_erected_fence_around = 
				Consequence.createElementaryConsequence(p61ErectedFenceAround,consequenceSet);
		
		// --- Create norm-set ---
		controlPanel.addNorm("n0", // a1 & b1
				Ground.createConjunctionOfGrounds(M1_owner_to_whiteacre,M1_owner_to_blackacre),
				Consequence.createConjunctionOfConsequences(P1_erected_fence_around,
//						Consequence.createConjunctionOfConsequences(P1_cows_entered_from_whiteacre_to_blackacre,
//								Consequence.createConjunctionOfConsequences(P1_cows_entered_from_blackacre_to_whiteacre,
										Consequence.createConjunctionOfConsequences(
												P1_main_building_on_whiteacre_painted_white,
												P1_main_building_on_blackacre_painted_white)));
		controlPanel.addNorm("n1", // a1 & -b1
				Ground.createConjunctionOfGrounds(M1_owner_to_whiteacre,not_M1_owner_to_blackacre),
				Consequence.createConjunctionOfConsequences(P41_erected_fence_around,
//						Consequence.createConjunctionOfConsequences(P7_cows_entered_from_whiteacre_to_blackacre,
//								Consequence.createConjunctionOfConsequences(P41_cows_entered_from_blackacre_to_whiteacre,
										Consequence.createConjunctionOfConsequences(
												P1_main_building_on_whiteacre_painted_white,
												P61_main_building_on_blackacre_painted_white)));
		controlPanel.addNorm("n2", // -a1 & b1
				Ground.createConjunctionOfGrounds(not_M1_owner_to_whiteacre,M1_owner_to_blackacre),
				Consequence.createConjunctionOfConsequences(P41_erected_fence_around,
//						Consequence.createConjunctionOfConsequences(P41_cows_entered_from_whiteacre_to_blackacre,
//								Consequence.createConjunctionOfConsequences(P7_cows_entered_from_blackacre_to_whiteacre,
										Consequence.createConjunctionOfConsequences(
												P61_main_building_on_whiteacre_painted_white,
												P1_main_building_on_blackacre_painted_white)));
		controlPanel.addNorm("n3", // -a1 & -b1
				Ground.createConjunctionOfGrounds(not_M1_owner_to_whiteacre,not_M1_owner_to_blackacre),
				Consequence.createConjunctionOfConsequences(P61_erected_fence_around,
//						Consequence.createConjunctionOfConsequences(P61_cows_entered_from_whiteacre_to_blackacre,
//								Consequence.createConjunctionOfConsequences(P61_cows_entered_from_blackacre_to_whiteacre,
										Consequence.createConjunctionOfConsequences(
												P61_main_building_on_whiteacre_painted_white,
												P61_main_building_on_blackacre_painted_white)));

		centerBox.add(controlPanel);
		/*
		 * --- SHOW PERMISSIBLE ACTS ---
		 */
		showPermissibleActsBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				Agent selectedAgent = controlPanel.getSelectedAgent();
				System.out.println("Permissible acts for " + selectedAgent);
				String title = "Permissible acts for " + selectedAgent;
				permissibleActs = controlPanel
						.getPermissibleActs(selectedAgent);
				String pActsString = GenericComponent.toString(permissibleActs);
				JOptionPane.showMessageDialog(centerBox, pActsString, title,
						JOptionPane.INFORMATION_MESSAGE);
			}
		});
		southBox.add(showPermissibleActsBtn);
		/*
		 * --- SHOW RESULTING STATE ---
		 */
		showResultingStateBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				Agent agent = controlPanel.getSelectedAgent();
				Act act = controlPanel.getSelectedAct();
				if (agent == null || act == null)
					JOptionPane.showMessageDialog(centerBox,
							"Please select agent and act", "Error",
							JOptionPane.INFORMATION_MESSAGE);
				else {
					String title = "Resulting state [" + agent + "-" + act
							+ "]";
					System.out.println(title);
					State state = controlPanel.getResultingState(agent, act);
					JOptionPane.showMessageDialog(centerBox, state.toString(),
							title, JOptionPane.PLAIN_MESSAGE);
				}
			}
		});
		southBox.add(showResultingStateBtn);

		centerBox.add(southBox);
		add(centerBox);
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				System.out.println("Inconsistency counter: "
						+ inconsistencyCounter);
				super.windowClosing(e);
			}
		});
		controlPanel.getController().addObserver(this);
		controlPanel.setAgentToMove(alice);
	}

	private List<Act> getFeasibleActs(Agent agent) {
		List<Act> feasibleActs = new LinkedList<Act>();		
		feasibleActs.add(Act.createAct("do_nothing"));
		feasibleActs.add(Act.createAct("paint_main_building_white",WHITEACRE));
		feasibleActs.add(Act.createAct("paint_main_building_black",WHITEACRE));
		feasibleActs.add(Act.createAct("erect_fence_around",
				controlPanel.splitParamsString("whiteacre,blackacre")));
//		feasibleActs.add(Act.createAct("let_cows_enter",
//				controlPanel.splitParamsString("whiteacre,blackacre")));
//		feasibleActs.add(Act.createAct("let_cows_enter",
//				controlPanel.splitParamsString("blackacre,whiteacre")));
//		if(Estate.containsStrayCattle("whiteacre",controlPanel.getCurrentState()))
//			feasibleActs.add(Act.createAct("remove_stray_cattle",WHITEACRE));
//		if(Estate.containsStrayCattle("blackacre",controlPanel.getCurrentState()))
//			feasibleActs.add(Act.createAct("remove_stray_cattle",BLACKACRE));
//			controlPanel.addAct("paint_main_building_black","whiteacre");
//			controlPanel.addAct("paint_main_building_black","blackacre");
//			controlPanel.addAct("paint_main_building_white","blueacre");			
//		}
//		controlPanel.setAgentToMove(agent);
		return feasibleActs;
	}
	
	private void setFeasibleActs(Agent selectedAgent, List<Act> acts) {
		controlPanel.clearActs();
		for(Act act : acts)
			controlPanel.addAct(act);
	}

	private void addEstate(String name, String ownerID, String mainBuilding,
			String cows, String fenced) {
		Agent agent = Agent.createAgent(ownerID);
		if (controlPanel.containsAgent(agent)) {
			PBTerm estateTerm = Estate.createEstateTerm(name,ownerID,mainBuilding,cows,fenced);
			List<PBTerm> stateTerms = State.createStateTermList(controlPanel
					.getCurrentState());
			stateTerms.add(estateTerm);
			controlPanel.setCurrentState(State.createState(stateTerms));
		} else
			JOptionPane.showMessageDialog(this, ownerID + ": no such agent",
					"Error", JOptionPane.ERROR_MESSAGE);
	}

	@Override
	public void update(Observable o, Object arg) {
		if(arg instanceof Agent) {
			Agent agent = (Agent)arg;
			List<Act> feasibleActs = getFeasibleActs(agent);
			setFeasibleActs(agent,feasibleActs);
		}
	}

}
