/*
 * Copyright 2013 Magnus Hjelmblom, University of Gvle, Sweden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package se.hig.nrts;

import java.util.LinkedList;
import java.util.List;
import java.util.Stack;

import se.sics.prologbeans.PBTerm;

public class Act extends GenericComponent {
	protected String module;
	protected String actID;
	protected String [] actParams;

//	/**
//	 * @param actID The name of this Act.
//	 */
//	protected Act(String actID) {
//		this(actID,null);
//	}

	/**
	 * @param module The Prolog module of this Act.
	 * @param actID The name (functor) of this Act.
	 * @param actParams The parameter list of this Act.
	 */
	protected Act(String module, String actID, String [] actParams) {
		this.module = module;
		this.actID = actID;
		this.actParams = actParams;
	}

	/**
	 * @return the Prolog module
	 */
	public String getModule() {
		return module;
	}

	/**
	 * @return the actID
	 */
	public String getActID() {
		return actID;
	}

	/**
	 * @return the actParam
	 */
	public String [] getActParams() {
		return actParams;
//		if (prologTerm.isCompound())
//			return prologTerm.getArgument(1).getName();
//		return null;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		if (this.actParams == null)
			return this.module + ":" + this.actID;
		return this.module + ":" + this.actID + "(" + toString(this.actParams) + ")";
	}
	
	
	@Override
	public int hashCode() {
		return toString().hashCode();
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		return this.toString().equals(obj.toString());
	}

	public static Act createAct(String actID) {
		return new Act(MOD_NRTS,actID,null);
	}
	
	public static Act createAct(String actID, String [] actParams) {
		return new Act(MOD_NRTS,actID,actParams);
	}

	public static Act createAct(String module, String actID, String [] actParams) {
		return new Act(module,actID,actParams);
	}

	public static PBTerm createActTerm(String module, String actID, String [] actParams) {
		if (actParams == null) {
			PBTerm [] actArgs = new PBTerm[2];
			actArgs[0] = PBTerm.makeAtom(module);
			actArgs[1] = PBTerm.makeAtom(actID);
			return PBTerm.makeTerm(":",actArgs);
		}
		else {
			PBTerm [] paramsTerm = new PBTerm[actParams.length];
			for(int i=0; i<paramsTerm.length; i++) {
				Long intParam = parseLong(actParams[i]);
				if (intParam != null)
					paramsTerm[i] = PBTerm.makeTerm(intParam);
				else {
					Double doubleParam = parseDouble(actParams[i]);
					if (doubleParam != null)
						paramsTerm[i] = PBTerm.makeTerm(doubleParam);
					else
						paramsTerm[i] = PBTerm.makeAtom(actParams[i]);
				}				
			}
			PBTerm [] actArgs = new PBTerm[2];
			actArgs[0] = PBTerm.makeAtom(module);
			actArgs[1] = PBTerm.makeTerm(actID,paramsTerm);
			return PBTerm.makeTerm(":",actArgs);
		}
	}

	private static Long parseLong(String s) {
		try {
			long l = Long.parseLong(s);
			return new Long(l);
		} catch (NumberFormatException nfe) {
			return null;
		}
	}

	private static Double parseDouble(String s) {
		try {
			double d = Double.parseDouble(s);
			return new Double(d);
		} catch (NumberFormatException nfe) {
			return null;
		}
	}

	public static Act createAct(PBTerm actTerm) {
		PBTerm moduleTerm = actTerm.getArgument(1);
		PBTerm actIDTerm = actTerm.getArgument(2);
		if (actIDTerm.isAtom())
			return new Act(moduleTerm.getName(),actIDTerm.getName(),null);
		String [] actParams = new String[actIDTerm.getArity()];
		for(int i=0; i<actParams.length; i++) {
			PBTerm paramTerm = actIDTerm.getArgument(i+1);
			actParams[i] = null;
			if (paramTerm.isInteger())
				actParams[i] = String.valueOf(paramTerm.intValue());
			else if (paramTerm.isFloat())
				actParams[i] = String.valueOf(paramTerm.floatValue());
			else
				actParams[i] = paramTerm.getName();			
		}
		return createAct(moduleTerm.getName(),actIDTerm.getName(),actParams);
	}
	
	public static PBTerm createActListPrologTerm(List<Act> actList) {
		Stack<Act> actStack = new Stack<Act>();
		for(Act act : actList)
			actStack.push(act);
		PBTerm actsTerm = PBTerm.NIL;
		while (!actStack.isEmpty()) {
			Act act = actStack.pop();
			actsTerm = PBTerm.makeTerm(createActTerm(act.module,act.actID,act.actParams),actsTerm);
//			term = PBTerm.makeTerm(act.getPrologTerm(),term);
		}
		return actsTerm;
	}

	public static List<Act> createActList(PBTerm actListTerm) {
		List<Act> actList = new LinkedList<Act>();
		while (!actListTerm.isEmptyList()) {
			Act act = createAct(actListTerm.head());
//			Act act = new Act(actListTerm.head());
			actList.add(act);
			actListTerm = actListTerm.tail();
		}
		return actList;
	}
	
	public static List<Act> createEmptyActList() {
		return new LinkedList<Act>();
	}

}