/*
 * Copyright 2013 Magnus Hjelmblom, University of Gvle, Sweden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package se.hig.nrts;

import se.hig.nrts.Act;
import se.hig.nrts.Agent;
import se.hig.nrts.PrologSessionHandler;
import se.hig.nrts.State;
import se.hig.nrts.exceptions.NRTSException;
import se.hig.nrts.norms.Consequence;
import se.hig.nrts.norms.Ground;
import se.hig.nrts.norms.Norm;
import se.hig.nrts.norms.NormSystem;
import se.hig.nrts.norms.Operator;
import se.hig.nrts.norms.SitCondition;
import se.hig.nrts.norms.StateCondition;

import java.util.List;
import java.util.Map;
import java.util.Observable;

public class NRTSController extends Observable {
	private NRTSSituation currentSituation;
	
	public NRTSController(String host, int port) {
		this(host,port,"_");
	}

	public NRTSController(String host, int port, String normSystemID) {
		if (port<0)
			throw new NRTSException("Error: illegal port value (" + port + ")");
		PrologSessionHandler.getInstance().createSession(host,port);
		currentSituation = NRTSSituation.createEmptySituation(normSystemID);
	}

	public void setCurrentState(State state) {
		currentSituation.setState(state);
	}

	public State getCurrentState() {
		return currentSituation.getState();
	}

	public void setAgentToMove(Agent agent) {
		System.out.println("Setting agent to move: " + agent);
		currentSituation.setAgentToMove(agent);
		setChanged();
		notifyObservers(agent);
	}

	public Agent getAgentToMove() {
		System.out.println("Agent to move: " + currentSituation.getAgentToMove());				
		return currentSituation.getAgentToMove();
	}

	public boolean containsAgent(Agent agent) {
		return currentSituation.getAgentSet().contains(agent);
	}

	public void addAgent(Agent agent) {
		if (containsAgent(agent))
			throw new NRTSException("Agent " + agent + " already exists");
		currentSituation.getAgentSet().add(agent);
	}

	public NRTSSituation getCurrentSituation() {
		return currentSituation;
	}

	public List<Agent> getAgentSet() {
		return currentSituation.getAgentSet();
	}

	public boolean containsAct(Act act) {
		return currentSituation.getActSet().contains(act);
	}

	public void clearActs() {
		currentSituation.getActSet().clear();
	}
	
	public void addAct(Act act) {
		if (containsAct(act))
			throw new NRTSException("Act " + act + ") already exists");
		currentSituation.getActSet().add(act);
	}

	public List<Act> getActSet() {
		return currentSituation.getActSet();
	}

	public void addConditionToGroundSet(String groundID, StateCondition stateCond) {
		if (groundID == null || groundID.equals(""))
			throw new NRTSException("Ground ID must not be empty");
		currentSituation.getNormSystem().addConditionToGroundSet(groundID,stateCond);
	}

	public String[] getGroundIDSet() {
		Map<String,StateCondition> groundSet = currentSituation.getNormSystem().getGroundSet();
		return groundSet.keySet().toArray(new String[groundSet.size()]);
	}

	
	public void addConditionToConsequenceSet(String consequenceID, StateCondition stateCond) {
		currentSituation.getNormSystem().addConditionToConsequenceSet(consequenceID,stateCond);
	}

	public String[] getConsequenceIDSet() {
		Map<String,StateCondition> consequenceSet = currentSituation.getNormSystem().getConsequenceSet();
		return consequenceSet.keySet().toArray(new String[consequenceSet.size()]);
	}

	public boolean containsNorm(NormSystem normSystem, String normID) {
		return normSystem.containsNorm(normID);
	}

	public void addNorm(String normID, Ground ground, Consequence consequence) {
		NormSystem normSystem = currentSituation.getNormSystem();
		if (normSystem.containsNorm(normID))
			throw new NRTSException("Norm ID must be unique");
		Norm norm = 
			NormSystem.createNorm(normID,ground,consequence);
		normSystem.addNorm(norm);
	}

	public void deleteNorm(String normID) {
		if (normID == null || normID.equals(""))
			throw new NRTSException("Norm ID must not be empty");
		currentSituation.getNormSystem().deleteNorm(normID);
	}
	
	public NormSystem getNormSystem() {
		return currentSituation.getNormSystem();
	}

	public List<Norm> getNormSet() {
		return currentSituation.getNormSystem().getNormSet();
	}

	public Map<String, StateCondition> getGroundSet() {
		return currentSituation.getNormSystem().getGroundSet();
	}

	public Map<String, StateCondition> getConsequenceSet() {
		return currentSituation.getNormSystem().getConsequenceSet();
	}

	public List<Act> getPermissibleActs(Agent agent) 
			throws NRTSException {
		return NRTSSituation.permissibleActs(agent,currentSituation);
	}
	
	public State getResultingState(Agent agent, Act act)
			throws NRTSException {
		return State.resultingState(agent,act,currentSituation.getState());
	}

}
