/*
 * Copyright 2013 Magnus Hjelmblom, University of Gvle, Sweden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package se.hig.nrts;

import java.io.IOException;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import se.hig.nrts.Act;
import se.hig.nrts.Agent;
import se.hig.nrts.exceptions.NRTSException;
import se.hig.nrts.norms.NormSystem;
import se.hig.nrts.norms.StateCondition;
import se.sics.prologbeans.Bindings;
import se.sics.prologbeans.IllegalCharacterSetException;
import se.sics.prologbeans.PBTerm;
import se.sics.prologbeans.QueryAnswer;

public class NRTSSituation extends GenericComponent {
	protected Agent agentToMove;
	protected State state;
	protected NormSystem normSystem;
	protected List<Agent> agents;
	protected List<Act> acts;

	/**
	 * @param agentToMove
	 * @param state
	 */
	protected NRTSSituation(Agent agentToMove, State state, NormSystem normSystem, List<Agent> agents, List<Act> acts) {
		this.agentToMove = agentToMove;
		this.state = state;
		this.normSystem = normSystem;
		this.agents = agents;
		this.acts = acts;
	}

	/**
	 * @return the agentToMove
	 */
	public Agent getAgentToMove() {
		return this.agentToMove;
	}

	/**
	 * @param agentToMove the agentToMove to set
	 */
	public void setAgentToMove(Agent agentToMove) {
		this.agentToMove = agentToMove;
	}

	public Agent findAgent(String agentID) {
		for (Agent agent : this.getAgentSet()) {
			if (agentID.equals(agent.getAgentID()))
				return agent;
		}
		return null;
	}

	public Act findAct(String actID, String [] actParameters) {
		for (Act act : this.getActSet()) {
			if (actID.equals(act.getActID()) && actParameters.equals(act.getActParams()))
				return act;
		}
		return null;
	}

	
	public State getState() {
		return this.state;
	}

	/**
	 * @param state the state to set
	 */
	public void setState(State state) {
		this.state = state;
	}

	public List<Agent> getAgentSet() {
		return this.agents;
	}
	
	/**
	 * @param agentSet the agentSet to set
	 */
	public void setAgentSet(List<Agent> agents) {
		this.agents = agents;
	}

	public List<Act> getActSet() {
		return this.acts;
	}

	/**
	 * @param actSet the actSet to set
	 */
	public void setActSet(List<Act> acts) {
		this.acts = acts;
	}

	public NormSystem getNormSystem() {
		return this.normSystem;
	}

	/**
	 * @param normSystem the normSystem to set
	 */
	public void setNormSystem(NormSystem normSystem) {
		this.normSystem = normSystem;
	}


	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "<" + this.agentToMove + "," + this.state.toString() + ">";
	}

	public static PBTerm createPrologTerm(NRTSSituation situation) {
		String agentToMove = situation.getAgentToMove().getAgentID();
		PBTerm [] args = {
				PBTerm.makeAtom(agentToMove),
				State.createPrologTerm(situation.getState()),
				NormSystem.createPrologTerm(situation.getNormSystem()),
				Act.createActListPrologTerm(situation.getActSet()),
				Agent.createAgentListPrologTerm(situation.getAgentSet())
		};
		return PBTerm.makeTerm(NRTS_SIT,args);
	}

	public static NRTSSituation createEmptySituation(String normSystemID) {
		return createSituation(null,null,NormSystem.createEmptyNormSystem(normSystemID),
				Act.createEmptyActList(),Agent.createEmptyAgentList());
	}

	public static NRTSSituation 
	createSituation(Agent agentToMove, State state, NormSystem normSystem, List<Act> acts, List<Agent> agents) {
		return new NRTSSituation(agentToMove,state,normSystem,agents,acts);
	}
	
	public static List<Act> permissibleActs(Agent agent, NRTSSituation situation) 
			throws NRTSException {
		final String A = "A";
		final String SIT = "Sit";
		final String PAs = "PermissibleActs";
		PBTerm actListTerm = null;
		try {
			Bindings b = new Bindings();
			b.bind(A,Agent.createAgentTerm(agent.getAgentID()));
			PBTerm situationTerm = createPrologTerm(situation);
			b.bind(SIT,situationTerm);
			System.err.println("?- " + b + ".");
			String queryString = PERMISSIBLE_ACTS + "(" + A + "," + SIT + "," + PAs + ")";
			System.err.println("?- " + queryString + ".");
			connect();
			QueryAnswer answer = 
				prologSessionHandler.getSession().executeQuery(queryString,b);
			System.err.println("=> " + answer);
			if ((answer == null) || answer.queryFailed()) {
				disconnect();
				return null;
			}
			actListTerm = answer.getValue(PAs);
			System.err.println("actListTerm = " + actListTerm);
		} catch(IOException x) {
			x.printStackTrace();
			throw new NRTSException(x.getMessage());
		} catch(IllegalCharacterSetException x) {
			x.printStackTrace();
			throw new NRTSException(x.getMessage());
		}
		disconnect();
		List<Act> permissibleActs = Act.createActList(actListTerm);
		return permissibleActs;
	}
	
}