/*
 * Copyright 2013 Magnus Hjelmblom, University of Gvle, Sweden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package se.hig.nrts;

import java.io.IOException;
import java.util.LinkedList;
import java.util.List;
import se.hig.nrts.exceptions.NRTSException;
import se.sics.prologbeans.IllegalCharacterSetException;
import se.sics.prologbeans.PBTerm;
import se.sics.prologbeans.QueryAnswer;

public class State extends GenericComponent {
	protected PBTerm prologTerm;

	protected State(PBTerm prologTerm) {
		this.prologTerm = prologTerm;
	}

	public PBTerm getPrologTerm() {
		return prologTerm;
	}

	public boolean isList() {
		return (prologTerm.isProperList() || prologTerm.isEmptyList());
	}

	public static State createState(PBTerm prologTerm) {
		return new State(prologTerm);
	}
	
	public static State createEmptyState() {
		return new State(PBTerm.NIL);
	}

	public static PBTerm createPrologTerm(State state) {
		return state.getPrologTerm();
	}
	
	public static List<PBTerm> createStateTermList(State state) {
		return createStateTermList(state.getPrologTerm());
	}

	public static List<PBTerm> createStateTermList(PBTerm stateTerm) {
		List<PBTerm> items = new LinkedList<PBTerm>();
		if(stateTerm.isEmptyList())
			return items;
		if(stateTerm.isListCell()) {
			while (!stateTerm.isEmptyList()) {
				items.add(stateTerm.head());
				stateTerm = stateTerm.tail();
				}
			return items;			
		}
		return null;
	}

	public static State createState(List<PBTerm> stateTerms) {
		return createState(createPrologTerm(stateTerms));
	}

	private static PBTerm createPrologTerm(List<PBTerm> stateTerms) {
		PBTerm stateTerm = PBTerm.NIL;
		List<PBTerm> reverseTerms = new LinkedList<PBTerm>();
		for(PBTerm term : stateTerms) {
			reverseTerms.add(0,term);
		}
		for(PBTerm term : reverseTerms)
			stateTerm = PBTerm.makeTerm(term,stateTerm);		
		return stateTerm;
	}

	public static State resultingState(Agent agent, Act act, State state) 
			throws NRTSException {
		final String SPLUS = "SPlus";
		QueryAnswer queryAnswer = null;
		PBTerm resultingStateTerm = null;
		try {
			String queryString = ACT_RESULT + "(" 
					+ act.toString() + "," 
					+ agent.toString() + "," 
					+ state.toString() + "," 
					+ SPLUS + ")";
//			System.out.println("?- " + queryString + ".");
			connect();
			queryAnswer = prologSessionHandler.getSession().executeQuery(queryString);
//			System.out.println("=> " + queryAnswer);
			if ((queryAnswer != null) && !queryAnswer.queryFailed()) {
				resultingStateTerm = queryAnswer.getValue(SPLUS);
//				System.out.println("=> " + resultingStateTerm);
				return State.createState(resultingStateTerm);
			}
		} catch(IOException ioe) {
			ioe.printStackTrace(System.err);
			throw new NRTSException(ioe.getMessage());
		} catch(IllegalCharacterSetException x) {
			x.printStackTrace(System.err);
			throw new NRTSException(x.getMessage());
		}
		disconnect();
		return null;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return toPrologString(prologTerm);
	}

	public static String toPrologString(PBTerm prologTerm) {
		if(!prologTerm.isCompound()) // Base cases
			return prologTerm.toString();
		StringBuffer sBuf = new StringBuffer();
		if(prologTerm.isListCell()) { // List:
			sBuf.append("[");
			while (!prologTerm.isEmptyList()) { // While end is not reached, 
				sBuf.append(toPrologString(prologTerm.head())+","); // convert to prolog string ...
				prologTerm = prologTerm.tail(); // ... and step forward
				}
			sBuf.deleteCharAt(sBuf.length()-1);
			sBuf.append("]");
			return sBuf.toString();
		}
		else { // Compound term:
			sBuf.append(prologTerm.getName() + "(");
			for(int i=1; i<=prologTerm.getArity(); i++) {
				sBuf.append(toPrologString(prologTerm.getArgument(i))+",");
			}
			sBuf.deleteCharAt(sBuf.length()-1);
			sBuf.append(")");
			return sBuf.toString();
		}
	}
	
}
