/*
 * Copyright 2013 Magnus Hjelmblom, University of Gvle, Sweden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package se.hig.nrts.norms;

import java.util.Map;

import se.hig.nrts.GenericComponent;
import se.hig.nrts.NRTSSituation;
import se.sics.prologbeans.PBTerm;

public class Consequence extends GenericComponent {
	protected Consequence leftOperand = null;
	protected String booleanConnective = null;
	protected Consequence rightOperand = null;
	protected SitCondition sitCond = null;

	/**
	 * Constructor for elementary consequences.
	 * 
	 * @param sitCond
	 */
	public Consequence(SitCondition sitCond) {
		this.sitCond = sitCond;
	}
	
	/**
	 * Constructor for non-elementary consequences.
	 * 
	 * @param leftOperand
	 * @param booleanConnective
	 * @param rightOperand
	 */
	public Consequence(Consequence leftOperand, String booleanConnective, Consequence rightOperand) {
		this.leftOperand = leftOperand;
		this.booleanConnective = booleanConnective;
		this.rightOperand = rightOperand;
	}
	
	public Consequence getLeftOperand() {
		return this.leftOperand;
	}

	public String getBooleanConnective() {
		return this.booleanConnective;
	}

	public Consequence getRightOperand() {
		return this.rightOperand;
	}

	public SitCondition getSitCond() {
		return this.sitCond;
	}

	public boolean isElementary() {
		return (this.booleanConnective == null);
	}
	
	public static PBTerm createPrologTerm(Consequence consequence) {
		if (consequence.isElementary())
			return SitCondition.createPrologTerm(consequence.getSitCond());
		Consequence leftOperand = consequence.getLeftOperand();
		String booleanConnective = consequence.getBooleanConnective();
		Consequence rightOperand = consequence.getRightOperand();
		PBTerm [] operands = {
				createPrologTerm(leftOperand),
				createPrologTerm(rightOperand)
		}; 
		return PBTerm.makeTerm(booleanConnective,operands);
	}

	public static Consequence createConsequence(PBTerm consequenceTerm) {
		String name = consequenceTerm.getName();
		if(isBooleanConnective(name)) {
			Consequence leftOperand = createConsequence(consequenceTerm.getArgument(0));
			Consequence rightOperand = createConsequence(consequenceTerm.getArgument(1));
			return new Consequence(leftOperand,name,rightOperand);
		}
		return new Consequence(SitCondition.createSitCond(consequenceTerm));
	}

	public static Consequence createElementaryConsequence(NRTSSituation currentSituation, 
			SitCondition sitCond, String [] consequenceConditionParameters) {
		NormSystem normSystem = currentSituation.getNormSystem();
		Map<String,StateCondition> consequenceSet = normSystem.getConsequenceSet();
		return createElementaryConsequence(sitCond,consequenceSet);
	}

	public static Consequence createElementaryConsequence(SitCondition sitCond, 
			Map<String, StateCondition> consequenceSet) {
		if (!consequenceSet.containsKey(sitCond.getSitCondID()))
			return null;
		return new Consequence(sitCond);
	}
	
	public static Consequence createConjunctionOfConsequences(Consequence c1, Consequence c2) {
		return new Consequence(c1,GenericComponent.AND,c2);
	}
	
	public static Consequence createDisjunctionOfConsequences(Consequence c1, Consequence c2) {
		return new Consequence(c1,GenericComponent.OR,c2);
	}
	
	@Override
	public String toString() {
		if(isElementary())
			return sitCond.toString();
		return "(" + leftOperand.toString() + booleanConnective + rightOperand.toString() + ")";
	}

}