/*
 * Copyright 2013 Magnus Hjelmblom, University of Gvle, Sweden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package se.hig.nrts.norms;

import java.util.Map;

import se.hig.nrts.GenericComponent;
import se.hig.nrts.NRTSSituation;
import se.sics.prologbeans.PBTerm;

public class Ground extends GenericComponent {
	protected Ground leftOperand = null;
	protected String booleanConnective = null;
	protected Ground rightOperand = null;
	protected SitCondition sitCond = null;
	
	/**
	 * Constructor for elementary grounds.
	 * 
	 * @param sitCond
	 */
	public Ground(SitCondition sitCond) {
		this.sitCond = sitCond;
	}
	
	/**
	 * Constructor for non-elementary grounds.
	 * 
	 * @param leftOperand
	 * @param booleanConnective
	 * @param rightOperand
	 */
	public Ground(Ground leftOperand, String booleanConnective, Ground rightOperand) {
		this.leftOperand = leftOperand;
		this.booleanConnective = booleanConnective;
		this.rightOperand = rightOperand;
	}

	public Ground(Ground leftOperand, String booleanConnective) {
		this.leftOperand = leftOperand;
		this.booleanConnective = booleanConnective;
	}

	public Ground getLeftOperand() {
		return this.leftOperand;
	}

	public String getBooleanConnective() {
		return this.booleanConnective;
	}

	public Ground getRightOperand() {
		return this.rightOperand;
	}

	public SitCondition getSitCond() {
		return this.sitCond;
	}

	public boolean isElementary() {
		return (this.booleanConnective == null);
	}
	
	public static PBTerm createPrologTerm(Ground ground) {
		if(ground.isElementary())
			return SitCondition.createPrologTerm(ground.getSitCond());
		Ground leftOperand = ground.getLeftOperand();
		String booleanConnective = ground.getBooleanConnective();
		if(isNegation(booleanConnective)) {
			PBTerm [] operands = { createPrologTerm(leftOperand) };
			return PBTerm.makeTerm(booleanConnective,operands);
		} // Binary operation
		Ground rightOperand = ground.getRightOperand();
		PBTerm [] operands = {
				createPrologTerm(leftOperand),
				createPrologTerm(rightOperand)
		}; 
		return PBTerm.makeTerm(booleanConnective,operands);
	}

	public static Ground createGround(PBTerm groundTerm) {
		String name = groundTerm.getName();
		if(isBooleanConnective(name)) {
			Ground leftOperand = createGround(groundTerm.getArgument(0));
			if(isNegation(name)) {
				return new Ground(leftOperand,name);
			} // Binary operation
			Ground rightOperand = createGround(groundTerm.getArgument(1));
			return new Ground(leftOperand,name,rightOperand);
		}
		return new Ground(SitCondition.createSitCond(groundTerm));
	}

	public static Ground createElementaryGround(SitCondition sitCond, 
			Map<String, StateCondition> groundSet) {
		if (!groundSet.containsKey(sitCond.getSitCondID()))
			return null;
		return new Ground(sitCond);
	}

	public static Ground createNegatedGround(Ground ground) {
		return new Ground(ground,GenericComponent.NOT);
	}

	public static Ground createConjunctionOfGrounds(Ground g1, Ground g2) {
		return new Ground(g1,GenericComponent.AND,g2);
	}

	public static Ground createDisjunctionOfGrounds(Ground g1, Ground g2) {
		return new Ground(g1,GenericComponent.OR,g2);
	}

	@Override
	public String toString() {
		if(isElementary())
			return sitCond.toString();
		if(isNegation(booleanConnective))
			return booleanConnective + "(" + leftOperand.toString() + ")";
		return "(" + leftOperand.toString() + booleanConnective + rightOperand.toString() + ")";
	}

}