/*
 * Copyright 2013 Magnus Hjelmblom, University of Gvle, Sweden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package se.hig.nrts.norms;

import se.hig.nrts.GenericComponent;
import se.hig.nrts.exceptions.NRTSException;
import se.sics.prologbeans.PBTerm;

public class SitCondition extends GenericComponent {
	protected Operator op;
	protected String sitCondID;
	protected int arity;
	protected String [] parameters;

	protected SitCondition(Operator op, String sitCondID, String[] sitConditionParameters) {
		this.op = op;
		this.sitCondID = sitCondID;
		if (sitConditionParameters == null) {
			this.arity = 0;
			this.parameters = null;
		}
		else {
			this.arity = sitConditionParameters.length;
			this.parameters = sitConditionParameters;
		}
	}
	
	/**
	 * @return the op
	 */
	public Operator getOp() {
		return op;
	}

	/**
	 * @return the sitCondID
	 */
	public String getSitCondID() {
		return sitCondID;
	}

	public int getArity() {
		return arity;
	}

	public String [] getParameters() {
		if (this.arity < 1)
			throw new NRTSException("Error: operator " + this.sitCondID + " has no parameters");
		return parameters;
	}

	public String toString() {
		if (this.arity < 1)
			return this.op + "(" + this.sitCondID + ")";
		return this.op + "(" + this.sitCondID + "(" + toString(this.parameters) + "))";
	}
	
	private static Long parseLong(String s) {
		try {
			long l = Long.parseLong(s);
			return new Long(l);
		} catch (NumberFormatException nfe) {
			return null;
		}
	}

	private static Double parseDouble(String s) {
		try {
			double d = Double.parseDouble(s);
			return new Double(d);
		} catch (NumberFormatException nfe) {
			return null;
		}
	}

	public static PBTerm createPrologTerm(SitCondition sitCond) {
		PBTerm opTerm = Operator.createPrologTerm(sitCond.getOp());
		PBTerm condTerm = null;
		if (sitCond.getArity() < 1)
			condTerm = PBTerm.makeAtom(sitCond.getSitCondID());
		else {
			PBTerm [] paramsTerm = new PBTerm[sitCond.getArity()];
			for(int i=0; i<paramsTerm.length; i++) {
				Long l = parseLong(sitCond.getParameters()[i]);
				if (l != null)
					paramsTerm[i] = PBTerm.makeTerm(l.longValue());
				else {
					Double d = parseDouble(sitCond.getParameters()[i]);
					if (d != null)
						paramsTerm[i] = PBTerm.makeTerm(d.doubleValue());
					else
						paramsTerm[i] = PBTerm.makeAtom(sitCond.getParameters()[i]);
				}				
			}
			condTerm = PBTerm.makeTerm(sitCond.getSitCondID(),paramsTerm);
		}
		PBTerm tail = PBTerm.makeTerm(condTerm,PBTerm.NIL);
		PBTerm head = PBTerm.makeTerm(opTerm,tail); // 2-item list
//		System.out.println(head);
		return head;
	}

	public static SitCondition createSitCond(PBTerm sitCondTerm) {
		Operator op = Operator.createOperator(sitCondTerm.head());
		PBTerm condTerm = sitCondTerm.tail();
		PBTerm condIDTerm = condTerm.head();
		if (condIDTerm.isCompound()) {
			String condID = condIDTerm.getName();
			String [] condParams = new String[condIDTerm.getArity()];
			for(int i=0; i<condParams.length; i++) {
				PBTerm paramTerm = condIDTerm.getArgument(i+1);
				if (paramTerm.isInteger())
					condParams[i] = String.valueOf(paramTerm.intValue());
				else if (paramTerm.isFloat())
					condParams[i] = String.valueOf(paramTerm.floatValue());
				else
					condParams[i] = paramTerm.getName();			
			}
			SitCondition sitCond = new SitCondition(op,condID,condParams);
			return sitCond;
		}
		String condID = condIDTerm.toString();
		return new SitCondition(op,condID,null);
	}

	public static SitCondition createSitCond(Operator opID,
			String sitCondID, String[] sitCondParameters) {
		return new SitCondition(opID,sitCondID,sitCondParameters);
	}
}