/*
 * Copyright 2013 Magnus Hjelmblom, University of Gvle, Sweden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package se.hig.nrts.server;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Arrays;

import se.hig.nrts.PrologSessionHandler;
import se.hig.nrts.exceptions.NRTSException;


public class NRTSServerDaemon {
	protected static final String LOCALHOST = "localhost";
	protected File sicstusPath = null;
	protected File serverFile = null;
	protected String serverStartupGoal = null;
	protected ServerThread serverThread = null;
	protected boolean serverThreadRunning = false;
	protected Process sicstusProcess = null;
	protected BufferedReader sicstusErrorStreamReader = null;
	protected BufferedReader sicstusOutputStreamReader = null;
	protected int timeout = 8000;

	/**
	 * @param sicstusPath
	 * @param serverFileName
	 * @param serverStartupGoal
	 * @param serverDirName
	 */
	public NRTSServerDaemon(File sicstusPath, File serverFile, String serverStartupGoal) {
		this.sicstusPath = sicstusPath;
		this.serverFile = serverFile;
		this.serverStartupGoal = serverStartupGoal;
		this.serverThread = new ServerThread();
	}

	public void start() {
		Thread t = new Thread(serverThread);
		t.setDaemon(true);
		t.start();
	}

	public void shutdown() {
		System.out.println("# Shutting down server... ");
		try {
			if (serverThread.normalShutdown())
				System.out.println("# Server shutdown OK!");
		} catch(InterruptedException ie) {
			System.err.println("# Server daemon interrupted: " + ie.getMessage());
		}
		System.out.println("# Destroying server process... ");
		sicstusProcess.destroy();
		System.out.println("# Server process destroyed!");
	}


	public boolean isRunning() {
		return this.serverThreadRunning;
	}


	/**
	 * @return the cfServerPort
	 */
	public int getPort() {
		int port = -1;
		try {
			port = serverThread.getPort();
		} catch(InterruptedException ie) {
			System.err.println("# Server daemon interrupted: " + ie.getMessage());
		}
		return port;
	}

	/**
	 * @return the timeout
	 */
	public int getTimeout() {
		return timeout;
	}


	/**
	 * @param timeout the timeout to set
	 */
	public void setTimeout(int timeout) {
		if (timeout < 0)
			throw new NRTSException("Error: timeout value must be positive");
		this.timeout = timeout;
	}


	@Override
	protected void finalize() throws Throwable {
		super.finalize();
		if (this.serverThreadRunning)
			shutdown();
	}

	protected class ServerThread implements Runnable {
		private int serverPort = -1;

		public void run() {
			System.out.print("# Server daemon is starting: ");
			serverThreadRunning = true;
			String [] startCommand = {
					sicstusPath.getPath(),
					"-l",
					serverFile.getPath(),
					"--goal",
					serverStartupGoal + "."
			};
//			String startCommand = sicstusPath + " -l " + serverFile 
//					+ " --goal " + serverStartupGoal + ".";
			System.out.println("# Starting server: " + Arrays.toString(startCommand) + "");
//			File workingDir = new File(serverFile.getParent() + "/\"");
			File workingDir = null;
			System.out.println("# Working directory: " + workingDir);
			try {
				sicstusProcess = 
					Runtime.getRuntime().exec(startCommand,null,workingDir);
				sicstusErrorStreamReader = new BufferedReader(
						new InputStreamReader(sicstusProcess.getErrorStream()));
				while(serverThreadRunning) {
					String errLine = sicstusErrorStreamReader.readLine();
					if (errLine != null)
						if(errLine.startsWith("port:")) {
							serverPort = Integer.parseInt(errLine.substring(5));
							System.out.println("# Server listening at port " + serverPort);
							synchronized(this) {
								notify();
							}
						}
						else
							System.err.println(errLine);
				}
			} catch (IOException ioe) {
				throw new NRTSException(ioe.getMessage());
			}
			System.out.println("# Server daemon has finished");				
		}

		public synchronized int getPort() throws InterruptedException {
			if (serverPort<0)
				wait(timeout);
			return serverPort;
		}

		public synchronized boolean normalShutdown() throws InterruptedException {
			wait(timeout); // Give server thread some time to finish up
			serverThreadRunning = false;
			PrologSessionHandler.getInstance().createSession(LOCALHOST,getPort());
			return PrologSessionHandler.getInstance().cleanShutdown();
		}
	}

}